import time

import machine
import bluetooth
import esp32

import lolin32 as board

import bthome
import scd40


def _linear(v: float, l0: float, h0: float, l1: float, h1: float) -> float:
    if v <= l0:
        return l1
    if v >= h0:
        return h1
    return l1 + (v - l0) * (h1 - l1) / (h0 - l0)


def _lipo_capacity(voltage: float) -> float:
    return _linear(voltage, 3.7, 4.30, 0.0, 1.0)


def run() -> None:
    print("Start")
    machine.freq(80_000_000)
    has_scd40 = board.HAS_SCD40

    if board.PS_ON:
        esp32.gpio_deep_sleep_hold(True)
        ps_on = machine.Pin(board.PS_ON, mode=machine.Pin.OUT, value=1, hold=True)

    if board.EXT_PWR_EN:
        ext_pwr_en = machine.Pin(
            board.EXT_PWR_EN, mode=machine.Pin.OUT, value=1, hold=True
        )

    print("ADC")
    vbat_s = machine.ADC(board.VBAT_S, atten=machine.ADC.ATTN_11DB)
    print("I2C")
    porta = machine.SoftI2C(
        scl=machine.Pin(board.PORTA_SCL),
        sda=machine.Pin(board.PORTA_SDA),
        freq=1_000_000,
    )

    if has_scd40 and machine.reset_cause() != machine.DEEPSLEEP_RESET:
        scd40.initialise(porta, 0x62)

    print("BLE")
    ble = bluetooth.BLE()

    while True:
        temperature, humidity, co2 = None, None, None

        if has_scd40:
            deadline = time.time() + 30
            while not scd40.is_ready(porta, 0x62):
                if time.time() >= deadline:
                    scd40.initialise(porta, 0x62)
                    deadline = time.time() + 30
                time.sleep(0.1)

            temperature, humidity, co2 = scd40.read(porta, 0x62)

        vbat = 0
        for i in range(10):
            vbat += vbat_s.read_uv()
        vbat = vbat * 1e-6 * board.VBAT_S_SCALE / 10
        print(vbat)

        ble.active(True)
        ble.gap_advertise(
            123_000,
            bthome.pack(
                name="enviro",
                temperature=temperature,
                humidity_ratio=humidity,
                co2=co2,
                battery_ratio=_lipo_capacity(vbat),
                battery_v=vbat,
            ),
        )
        time.sleep(1)
        print("Sleep")
        machine.deepsleep(30_000)


def main():
    while True:
        try:
            run()
        except Exception as ex:
            print("Exception", ex)
            time.sleep(1)
        finally:
            machine.reset()


if __name__ == "__main__":
    main()
